@@ -782,10 +782,10 @@ class BaseModel(metaclass=_model_construction.ModelMetaclass):
         }
 
     def __setstate__(self, state: dict[Any, Any]) -> None:
-        _object_setattr(self, '__dict__', state['__dict__'])
         _object_setattr(self, '__pydantic_fields_set__', state['__pydantic_fields_set__'])
         _object_setattr(self, '__pydantic_extra__', state['__pydantic_extra__'])
         _object_setattr(self, '__pydantic_private__', state['__pydantic_private__'])
+        _object_setattr(self, '__dict__', state['__dict__'])
 
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, BaseModel):
@@ -835,6 +835,9 @@ class BaseModel(metaclass=_model_construction.ModelMetaclass):
     def __iter__(self) -> TupleGenerator:
         """So `dict(model)` works."""
         yield from self.__dict__.items()
+        extra = self.__pydantic_extra__
+        if extra:
+            yield from extra.items()
 
     def __repr__(self) -> str:
         return f'{self.__repr_name__()}({self.__repr_str__(", ")})'
