@@ -3,50 +3,50 @@ Test pydantic's compliance with mypy.
 
 Do a little skipping about with types to demonstrate its usage.
 """
-import json
 import os
-from datetime import date, datetime, timedelta
+from datetime import date, datetime, timedelta, timezone
 from pathlib import Path, PurePath
-from typing import Any, Dict, ForwardRef, Generic, List, Optional, TypeVar
+from typing import Any, Dict, ForwardRef, Generic, List, Optional, Type, TypeVar
 from uuid import UUID
 
-from typing_extensions import TypedDict
+from typing_extensions import Annotated, TypedDict
 
 from pydantic import (
     UUID1,
-    BaseConfig,
+    AwareDatetime,
     BaseModel,
-    BaseSettings,
+    ConfigDict,
     DirectoryPath,
-    Extra,
     FilePath,
     FutureDate,
+    FutureDatetime,
+    ImportString,
     Json,
+    NaiveDatetime,
     NegativeFloat,
     NegativeInt,
-    NoneStr,
     NonNegativeFloat,
     NonNegativeInt,
     NonPositiveFloat,
     NonPositiveInt,
     PastDate,
+    PastDatetime,
     PositiveFloat,
     PositiveInt,
-    PyObject,
     StrictBool,
     StrictBytes,
     StrictFloat,
     StrictInt,
     StrictStr,
+    UrlConstraints,
     create_model,
-    create_model_from_typeddict,
+    field_validator,
     root_validator,
-    stricturl,
-    validate_arguments,
-    validator,
+    validate_call,
 )
 from pydantic.fields import Field, PrivateAttr
-from pydantic.generics import GenericModel
+from pydantic.json_schema import Examples
+from pydantic.networks import AnyUrl
 
 
 class Flags(BaseModel):
@@ -58,21 +58,21 @@ class Flags(BaseModel):
 
 class Model(BaseModel):
     age: int
-    first_name = 'John'
-    last_name: NoneStr = None
+    first_name: str = 'John'
+    last_name: Optional[str] = None
     signup_ts: Optional[datetime] = None
     list_of_ints: List[int]
 
-    @validator('age')
+    @field_validator('age')
     def check_age(cls, value: int) -> int:
         assert value < 100, 'too old'
         return value
 
-    @root_validator
+    @root_validator(skip_on_failure=True)
     def root_check(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         return values
 
-    @root_validator(pre=True, allow_reuse=False, skip_on_failure=False)
+    @root_validator(pre=True, allow_reuse=False)
     def pre_root_check(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         return values
 
@@ -85,7 +85,7 @@ def day_of_week(dt: datetime) -> int:
     return dt.date().isoweekday()
 
 
-m = Model(age=21, list_of_ints=[1, '2', b'3'])
+m = Model(age=21, list_of_ints=[1, 2, 3])
 
 assert m.age == 21, m.age
 m.age = 42
@@ -98,7 +98,16 @@ dog_age = dog_years(m.age)
 assert dog_age == 294, dog_age
 
 
-m = Model(age=2, first_name=b'Woof', last_name=b'Woof', signup_ts='2017-06-07 00:00', list_of_ints=[1, '2', b'3'])
+Model(age=2, first_name='Woof', last_name='Woof', signup_ts=datetime(2017, 6, 7), list_of_ints=[1, 2, 3])
+m = Model.model_validate(
+    {
+        'age': 2,
+        'first_name': b'Woof',
+        'last_name': b'Woof',
+        'signup_ts': '2017-06-07 00:00',
+        'list_of_ints': [1, '2', b'3'],
+    }
+)
 
 assert m.first_name == 'Woof', m.first_name
 assert m.last_name == 'Woof', m.last_name
@@ -107,7 +116,7 @@ assert day_of_week(m.signup_ts) == 3
 
 
 data = {'age': 10, 'first_name': 'Alena', 'last_name': 'Sousova', 'list_of_ints': [410]}
-m_from_obj = Model.parse_obj(data)
+m_from_obj = Model.model_validate(data)
 
 assert isinstance(m_from_obj, Model)
 assert m_from_obj.age == 10
@@ -115,17 +124,9 @@ assert m_from_obj.first_name == data['first_name']
 assert m_from_obj.last_name == data['last_name']
 assert m_from_obj.list_of_ints == data['list_of_ints']
 
-m_from_raw = Model.parse_raw(json.dumps(data))
+m_copy = m_from_obj.model_copy()
 
-assert isinstance(m_from_raw, Model)
-assert m_from_raw.age == m_from_obj.age
-assert m_from_raw.first_name == m_from_obj.first_name
-assert m_from_raw.last_name == m_from_obj.last_name
-assert m_from_raw.list_of_ints == m_from_obj.list_of_ints
-
-m_copy = m_from_obj.copy()
-
-assert isinstance(m_from_raw, Model)
+assert isinstance(m_copy, Model)
 assert m_copy.age == m_from_obj.age
 assert m_copy.first_name == m_from_obj.first_name
 assert m_copy.last_name == m_from_obj.last_name
@@ -135,7 +136,7 @@ assert m_copy.list_of_ints == m_from_obj.list_of_ints
 T = TypeVar('T')
 
 
-class WrapperModel(GenericModel, Generic[T]):
+class WrapperModel(BaseModel, Generic[T]):
     payload: T
 
 
@@ -154,11 +155,11 @@ assert model_instance.payload.list_of_ints == [1, 2, 3, 4]
 
 class WithField(BaseModel):
     age: int
-    first_name: str = Field('John', const=True)
+    first_name: str = Field('John', max_length=42)
 
 
 # simple decorator
-@validate_arguments
+@validate_call
 def foo(a: int, *, c: str = 'x') -> str:
     return c * a
 
@@ -168,7 +169,7 @@ foo(1)
 
 
 # nested decorator should not produce an error
-@validate_arguments(config={'arbitrary_types_allowed': True})
+@validate_call(config={'arbitrary_types_allowed': True})
 def bar(a: int, *, c: str = 'x') -> str:
     return c * a
 
@@ -185,11 +186,11 @@ FooRef = ForwardRef('Foo')
 
 
 class MyConf(BaseModel):
-    str_pyobject: PyObject = Field('datetime.date')
-    callable_pyobject: PyObject = Field(date)
+    str_pyobject: ImportString[Type[date]] = Field(...)
+    callable_pyobject: ImportString[Type[date]] = Field(default=date)
 
 
-conf = MyConf()
+conf = MyConf(str_pyobject='datetime.date')
 var1: date = conf.str_pyobject(2020, 12, 20)
 var2: date = conf.callable_pyobject(2111, 1, 1)
 
@@ -199,6 +200,8 @@ class MyPrivateAttr(BaseModel):
 
 
 class PydanticTypes(BaseModel):
+    model_config = ConfigDict(validate_default=True)
+
     # Boolean
     my_strict_bool: StrictBool = True
     # Integer
@@ -217,33 +220,34 @@ class PydanticTypes(BaseModel):
     my_strict_bytes: StrictBytes = b'pika'
     # String
     my_strict_str: StrictStr = 'pika'
-    # PyObject
-    my_pyobject_str: PyObject = 'datetime.date'  # type: ignore
-    my_pyobject_callable: PyObject = date
+    # ImportString
+    import_string_str: ImportString[Any] = 'datetime.date'  # type: ignore[misc]
+    import_string_callable: ImportString[Any] = date
     # UUID
     my_uuid1: UUID1 = UUID('a8098c1a-f86e-11da-bd1a-00112444be1e')
-    my_uuid1_str: UUID1 = 'a8098c1a-f86e-11da-bd1a-00112444be1e'  # type: ignore
+    my_uuid1_str: UUID1 = 'a8098c1a-f86e-11da-bd1a-00112444be1e'
     # Path
     my_file_path: FilePath = Path(__file__)
-    my_file_path_str: FilePath = __file__  # type: ignore
+    my_file_path_str: FilePath = __file__
     my_dir_path: DirectoryPath = Path('.')
-    my_dir_path_str: DirectoryPath = '.'  # type: ignore
+    my_dir_path_str: DirectoryPath = '.'
     # Json
-    my_json: Json[Dict[str, str]] = '{"hello": "world"}'  # type: ignore
-    my_json_list: Json[List[str]] = '["hello", "world"]'  # type: ignore
+    my_json: Json[Dict[str, str]] = '{"hello": "world"}'
+    my_json_list: Json[List[str]] = '["hello", "world"]'
     # Date
     my_past_date: PastDate = date.today() - timedelta(1)
     my_future_date: FutureDate = date.today() + timedelta(1)
-
-    class Config:
-        validate_all = True
+    # Datetime
+    my_past_datetime: PastDatetime = datetime.now() - timedelta(1)
+    my_future_datetime: FutureDatetime = datetime.now() + timedelta(1)
+    my_aware_datetime: AwareDatetime = datetime.now(tz=timezone.utc)
+    my_naive_datetime: NaiveDatetime = datetime.now()
 
 
 validated = PydanticTypes()
-validated.my_pyobject_str(2021, 1, 1)
-validated.my_pyobject_callable(2021, 1, 1)
+validated.import_string_str(2021, 1, 1)
+validated.import_string_callable(2021, 1, 1)
 validated.my_uuid1.hex
-validated.my_uuid1_str.hex
 validated.my_file_path.absolute()
 validated.my_file_path_str.absolute()
 validated.my_dir_path.absolute()
@@ -251,9 +255,15 @@ validated.my_dir_path_str.absolute()
 validated.my_json['hello'].capitalize()
 validated.my_json_list[0].capitalize()
 
-stricturl(allowed_schemes={'http'})
-stricturl(allowed_schemes=frozenset({'http'}))
-stricturl(allowed_schemes=('s3', 's3n', 's3a'))
+
+class UrlModel(BaseModel):
+    x: Annotated[AnyUrl, UrlConstraints(allowed_schemes=['http'])] = Field(default=None)
+    y: Annotated[AnyUrl, UrlConstraints(allowed_schemes=['http'])] = Field(default=None)
+    z: Annotated[AnyUrl, UrlConstraints(allowed_schemes=['s3', 's3n', 's3a'])] = Field(default=None)
+
+
+url_model = UrlModel(x='http://example.com')
+assert url_model.x.host == 'example.com'
 
 
 class SomeDict(TypedDict):
@@ -267,14 +277,7 @@ obj: SomeDict = {
 }
 
 
-class Config(BaseConfig):
-    title = 'Record'
-    extra = Extra.ignore
-    max_anystr_length = 1234
-
-
-class Settings(BaseSettings):
-    ...
+config = ConfigDict(title='Record', extra='ignore', str_max_length=1234)
 
 
 class CustomPath(PurePath):
@@ -285,10 +288,6 @@ class CustomPath(PurePath):
         return f'a/custom/{self.path}'
 
 
-def dont_check_path_existence() -> None:
-    Settings(_env_file='a/path', _secrets_dir='a/path')
-    Settings(_env_file=CustomPath('a/path'), _secrets_dir=CustomPath('a/path'))
-
-
-create_model_from_typeddict(SomeDict)(**obj)
 DynamicModel = create_model('DynamicModel')
+
+examples = Examples({})
