@@ -279,6 +279,26 @@ def test_include_exclude():
     assert m.model_dump(exclude={'x': ..., 'y_list': {2}}) == {'y': 2, 'x_list': [1, 2], 'y_list': [2, 3]}
 
 
+def test_exclude_none():
+    class Model(BaseModel):
+        x: int
+        y: int
+
+        @computed_field
+        def sum(self) -> int:
+            return self.x + self.y
+
+        @computed_field
+        def none(self) -> None:
+            return None
+
+    m = Model(x=1, y=2)
+    assert m.model_dump(exclude_none=False) == {'x': 1, 'y': 2, 'sum': 3, 'none': None}
+    assert m.model_dump(exclude_none=True) == {'x': 1, 'y': 2, 'sum': 3}
+    assert m.model_dump(mode='json', exclude_none=False) == {'x': 1, 'y': 2, 'sum': 3, 'none': None}
+    assert m.model_dump(mode='json', exclude_none=True) == {'x': 1, 'y': 2, 'sum': 3}
+
+
 def test_expected_type():
     class Model(BaseModel):
         x: int
@@ -383,7 +403,7 @@ def test_private_computed_field():
     assert m.model_dump() == {'x': 2, '_double': 4}
 
 
-@pytest.mark.skipif(sys.version_info < (3, 9), reason='fails before 3.9 - Do we want to fix this???')
+@pytest.mark.skipif(sys.version_info < (3, 9), reason='@computed_field @classmethod @property only works in 3.9+')
 def test_classmethod():
     class MyModel(BaseModel):
         x: int
