@@ -8,8 +8,8 @@ git fetch --tags
 
 # TODO: Use the proper latest tag once FastAPI stable release is compatible with Pydantic V2.
 # latest_tag=$(git describe --tags --abbrev=0)
-latest_tag="main-pv2"
-git checkout "${latest_tag}"
+# git switch -d "${latest_tag}"
+git switch "main-pv2"
 
 pip install -r requirements.txt
 
