@@ -1029,6 +1029,21 @@ def test_model_iteration():
     assert dict(m) == {'c': 3, 'd': Foo()}
 
 
+def test_model_iteration_extra() -> None:
+    class Foo(BaseModel):
+        x: int = 1
+
+    class Bar(BaseModel):
+        a: int
+        b: Foo
+        model_config = ConfigDict(extra='allow')
+
+    m = Bar.model_validate({'a': 1, 'b': {}, 'c': 2, 'd': Foo()})
+    assert m.model_dump() == {'a': 1, 'b': {'x': 1}, 'c': 2, 'd': {'x': 1}}
+    assert list(m) == [('a', 1), ('b', Foo()), ('c', 2), ('d', Foo())]
+    assert dict(m) == {'a': 1, 'b': Foo(), 'c': 2, 'd': Foo()}
+
+
 @pytest.mark.parametrize(
     'exclude,expected,raises_match',
     [
@@ -2679,3 +2694,32 @@ def test_super_delattr_private():
     assert test_calls == []
     del m.test
     assert test_calls == ['success']
+
+
+def test_arbitrary_types_not_a_type() -> None:
+    """https://github.com/pydantic/pydantic/issues/6477"""
+
+    class Foo:
+        pass
+
+    class Bar:
+        pass
+
+    with pytest.warns(UserWarning, match='is not a Python type'):
+        ta = TypeAdapter(Foo(), config=ConfigDict(arbitrary_types_allowed=True))
+
+    bar = Bar()
+    assert ta.validate_python(bar) is bar
+
+
+def test_deferred_core_schema() -> None:
+    class Foo(BaseModel):
+        x: 'Bar'
+
+    with pytest.raises(PydanticUserError, match='`Foo` is not fully defined'):
+        Foo.__pydantic_core_schema__
+
+    class Bar(BaseModel):
+        pass
+
+    assert Foo.__pydantic_core_schema__
